#!/usr/bin/perl
#
# Script: tkvalid.pl - perl/tk front end to the PDS Validation Tools
# Usage:  tkvalid.pl [-d]
#
# Author: Joel Wilf
# Date: 08-29-99
#
# Description:
#   This script is a perl/tk front end the PDS Validataion Tools.
#   It gets user input through dialog windows and runs the appropriate
#   validation tests automatically. If the -d option is set, it will
#   run in debug mode and only print out the volume root and tests
#   selected. NOTE: tkvalid.pl requires that the user's perl 
#   installation include the Tk module.
# End:
#
# Revisions:
#   2000-04-03, J. Wilf, Added ISOCHECK and KWVTOOL checks.
#   2002-03-11, J. Wilf, Changed default directory to current one
#   2003-04-29, M. Cayanan, Added another button called "Change Label
#     Validation Settings" to allow user to modify certain parameters
#     if the Label Validation test is run. Also took out any reference
#     to checklab test since lvtool already does this through the 
#     label validation test.
#   2003-09-19, M. Cayanan, Carried over code to display the Label 
#     Validation Settings GUI. This was previously stored in the 
#     validate.pl script, but would not be able to pop up unless the user
#     hit "Run Tests" on the screen. So, by transferring all code to this
#     script file, the GUI will be able to show once the user wants to change
#     the label validation test settings
#
# End:


use Tk;
use Tk::LabEntry;
use Cwd;
use Getopt::Std;
use File::Basename;
$| = 1;

### Check for debug flag
getopts('d');  # If debug flag is set, $opt_d == 1
  
# Save current directory but run script from PVT home directory
$current_directory = getcwd();
$home_pvt = dirname($0);
chdir $home_pvt or die "Can\'t run from the PVT home directory\n";

### Create main window
$mw = MainWindow->new();
$mw->title("PDS Validation Tools Version 2.0");

# Help text for path selection
$explain_path_selection = <<"EOF1";
Enter the path to the root directory on the volume you want to
validate. Alternatively, you can press the 'BROWSE...' button
and open any file on your volume's root directory.
EOF1

# Help text for test selection
$explain_test_selection = <<"EOF2";
Select any of the following tests to run. The 'Choose/Clear All'
button will select all the tests (unless all the tests are
already selected, in which case it will unselect all the tests).
EOF2

### Frame 1 is for obtaining the volume root

$frame1 = $mw->Frame(
                     -borderwidth => 2,
                     -relief => 'groove'
                    )->pack(
                    	    -anchor => 'nw',
                            -padx => 10,
                            -pady => 5
                            );

$frame1->Label(
               -font => "Helvetica 18 normal",
               -text => "Set Path to Volume Root"
              )->pack(
                      -anchor => 'w'
                     );
                     
$frame1->Label(
               -text => "$explain_path_selection",
               -justify => 'left'
              )->pack(
                      -anchor => 'w'
                     );

$frame1->Entry(
               -background => 'white',
               -textvariable => \$root_dir
              )->pack(
                      -side => 'left',
                      -padx => 5
                     );
              
$root_dir = "e:\\"; # Default root directory

$frame1->Button(
                -text => "Browse...",
                -command => \&get_root_dir
               )->pack(
                       -side => 'left',
                       -pady => 5
                     );

### Frame 2 is for choosing the tests to run

$frame2 = $mw->Frame(
                     -borderwidth => 2,
                     -relief => 'groove'
                    )->pack(
                            -anchor => 'w',
                            -padx => 10,
                            -pady => 5,
                            -expand => 1,
                            -fill => 'x'
                           );

$frame2->Label(
               -font => "Helvetica 18 normal",
               -text => "Choose Tests to Run",
              )->pack(
                      -anchor => 'w'
                     );
                 
$frame2->Label(
               -text => "$explain_test_selection",
               -justify => 'left'
              )->pack(
                      -anchor => 'w'
                     );

$frame2->Button(
                -text => "Choose/Clear All",
                -command => \&choose_all,
                )->pack(
                        -anchor => 'w',
                        -padx => 5
                       );


if( ($^O =~ m/linux/) || ($^O =~ m/solaris/) ) {

    $frame2->Checkbutton(
                         -text => "Check ISO9660 compliance",
                         -variable => \$checkiso,
                         -command => \&tbox_state,
                         )->pack(
                                 -anchor => 'w'
                                );
}
else {
    $frame2->Checkbutton(
                         -text => "Check ISO9660 compliance",
                         -variable => \$checkiso
                        )->pack(
                                -anchor => 'w'
                               );
}

if( ($^O =~ m/linux/) || ($^O =~ m/solaris/) ) {

     $frame2->Label(
                    -text => "Enter the path to the cdrom device below: (i.e. /dev/cdrom)",
                    -justify => 'left'
                   )->pack(
                           -anchor => 'w'
                          );

     $tbox = $frame2->Entry(
                    -background => 'light gray',
                    -textvariable => \$cd_dir,
                    -state => 'disabled'
                    )->pack(
                            -anchor => 'w',
                            -padx => 5
                           );
}

$frame2->Checkbutton(
                     -text => "Check for required files",
                     -variable => \$checkreq
                    )->pack(
                            -anchor => 'w'
                           );

# 04-21-03 MDC - This feature is built in to the lvtool already.
# Select the "Change label validation settings" box to enable this 
# feature.

#$frame2->Checkbutton(
#                     -text => "Check that every file has a label",
#                     -variable => \$checklab
#                    )->pack(
#                            -anchor => 'w',
#                           );

#$frame2->Checkbutton(
#                     -text => "List label validation errors",
#                     -variable => \$listerr,
#                     -command => \&settings_state
#                    )->pack(
#                            -anchor => 'w'
#                           );

$cb = $frame2->Checkbutton(
		   -text => "Change Label Validation Settings",
		   -variable => \$listerr,
		   -command => \&disp_lvt_settings
		   )->pack(
		   	   -anchor => 'w'
		      );

$frame2->Checkbutton(
                     -text => "List new keywords",
                     -variable => \$listnew
                     )->pack(
                             -anchor => 'w'
                            );

$frame2->Checkbutton(
                     -text => "Check the index files",
                     -variable => \$checkidx
                    )->pack(
                            -anchor => 'w'
                           );

$frame2->Checkbutton(
                     -text => "Test formatting in ASCII files",
                     -variable => \$testline
                    )->pack(
                            -anchor => 'w'
                           );

$frame2->Checkbutton(
                     -text => "Check references",
                     -variable => \$checkref
                    )->pack(
                            -anchor => 'w'
                           );

$frame2->Checkbutton(
                     -text => "List keyword/value pairs",
                     -variable => \$listkwv
                    )->pack(
                            -anchor => 'w'
                           );

### Frame 3 run the tests or exits

$frame3 = $mw->Frame(
                     -borderwidth => 2,
                    )->pack(
                            -anchor => 'w',
                            -padx => 10,
                            -expand => 1,
                            -fill => 'x'             
                           );

$frame3->Button(
            -text => "Run Tests",
            -command => sub {
            	if($listerr == 1) {
            		set_lvt_options();
            	}
            	run_tests();
              }
               )->pack(
                   -side => 'left'
                  );

$frame3->Button(
                -text => "Exit",
                -command => sub { exit }
                )->pack(
                        -side => 'right'
                       );

MainLoop;

###########################
### Support Subroutines ###
###########################

# Get the root directory through a file-browsing dialog
sub get_root_dir {

    $file = $frame1->getOpenFile();
    $file =~ s#\/[^/]+$#\/#; # Get directory from filename
    $file = lc $file if ($file =~ m/\w:/); # Another evil Windows hack
    $root_dir = $file;
}

# Choose or clear all the checkboxes
sub choose_all {
	
    # Get the sum of all the tests selected
    $first5 = $checkiso + $checkreq + $checklab + $listerr + $listnew;
    $last4 = $checkidx + $testline + $checkref + $listkwv;
    $all = 0 + $first5 + $last4;
    
    # If all the tests are selected, clear them
    if ($all == 9) {
        $checkiso = 0;
        $checkreq = 0;
        $checklab = 0;
        $listerr = 0;
        $listnew = 0;
        $checkidx = 0;
        $testline = 0;
        $checkref = 0;
        $listkwv = 0;
    # Otherwise, select all the tests
    } else {
        $checkiso = 1;
        $checkreq = 1;
        $checklab = 1;
        $listerr = 1;
        $listnew = 1;
        $checkidx = 1;
        $testline = 1;
        $checkref = 1;
        $listkwv = 1;
    }
    
    disp_lvt_settings();
    
    if( ($^O =~ m/solaris/) || ($^O =~ m/linux/) ) {
    	tbox_state();
    }
}

# Run the validation tests
sub run_tests {
	
    # Get the sum of all the tests selected
    $first5 = $checkiso + $checkreq + $checklab + $listerr + $listnew;
    $last4 = $checkidx + $testline + $checkref + $listkwv;
    $all = 0 + $first5 + $last4;
    
    # If the volume root directory can't be found, give error dialog
    unless (-d $root_dir) {
        &error_dialog("Can\'t open volume root: $root_dir");
        return;
    }
    
    # If no tests are specified, give error dialog
    if ($all == 0) {
        &error_dialog("No tests chosen");
        return;
    }
    
    # If validate.pl isn't present, give error dialog
    unless (-e "validate.pl") {
        &error_dialog("Can\'t find validate.pl");
    }
    
    if( ($^O =~ m/linux/) || ($^O =~ m/solaris/) ) {
    	if( ($checkiso == 1) && ($cd_dir eq "") ) {
    	   &error_dialog("Path to the cdrom device not specified");
    	   return;
    	}
    }
    $mw->withdraw();
    
    if($tl != NULL) {
    	$tl->destroy();
    	$tl = "";
        cleanup_tl_vars();
    }   
    # 04-21-03 MDC - Since we've taken out the checklab option,
    # bump up the numbers of the commands that follow. 
    # i.e. $listerr was option 4, now it is option 3.
    
    # Build the command string and run it
    my $command = "perl validate.pl -";
    $command = $command . "1" if ($checkiso == 1);
    $command = $command . "2" if ($checkreq == 1);
 #  $command = $command . "3" if ($checklab == 1);
    $command = $command . "3" if ($listerr == 1);
    $command = $command . "4" if ($listnew == 1);
    $command = $command . "5" if ($checkidx == 1);
    $command = $command . "6" if ($testline == 1);
    $command = $command . "7" if ($checkref == 1);
    $command = $command . "8" if ($listkwv == 1);
    $command = "perl validate.pl -b" if ($all == 9);
    
    if ( $listerr ==1 ) {
    	    $command = $command . " --lvt" . " $command_opts";
    	    $cb->deselect();
    }
    
    $command = $command . " " . "-c" . " " . $cd_dir if ( ($checkiso == 1) && (($^O =~ m/linux/) || ($^O =~ m/solaris/)) );
    $command = $command . " " . $root_dir . " " . $current_directory;
    print "Command = $command\n" if ($opt_d);
    system("$command") unless ($opt_d);
    $mw->deiconify();
    $mw->raise();
}

# Create error dialog
sub error_dialog {

    my $message = shift @_;
    
    $mw->Dialog(
                -title => 'ERROR',
                -text => "$message",
                -bitmap => 'error',
                -default_button => 'OK',
                -buttons        => ['OK']
               )->Show();
               
}

# This routine is only used for non-Windows machines because of the 
# need for an extra field to properly do iso_checking in Solaris
# and Linux.
sub tbox_state {
    if($checkiso) {
         $tbox->configure(-background => 'white',
			  -state => 'normal'
			  );
     }
    else {
	$tbox->configure(-background => 'light gray',
			 -state => 'disabled'
			);
	$cd_dir="";
    }
}


# Display a GUI which shows all the Warning and Error messages that can
# appear in a label validation test report. Also give the user the ability
# to enable/disable certain lvtool options.

sub disp_lvt_settings {
   
   if($listerr == 0) {
   	if($tl != NULL) {
   		$tl->destroy();
   		$tl ="";
   	}
      	return;
   }
   else {
   	if($tl != NULL) {
   		$tl->deiconify();
   		$tl->raise();
   		return;
   	}
   }
   
   $clear_logfile = 0;
   $no_info = 0;
   $no_warnings = 0;
   
   ### Create main window
   $tl = $mw->Toplevel();
   $tl->title("Label Validation Settings");
    
   $logfile = "logfile";
 
$explain_error_selection = <<"EOF1";
Select the types of error messages that you would like the label validation 
test to ignore. Holding down the cntl key while clicking on the 
messages will allow you to make multiple selections.
EOF1

$explain_warning_selection = <<"EOF2";
Select the types of warning messages that you would like the label validation test to ignore. 
Holding down the cntl key while clicking on the messages will allow you to make multiple selections. 
EOF2
   
   # Display listbox with warning messages
   
   $frame2_tl = $tl->Frame(
                     -borderwidth => 4,
                     -relief => 'groove'
                    )->pack(
                            -padx => 5,
                            -pady => 5,
                            -side => 'top',
                            -fill => 'both'
                           );


   # Frame 1 to display list box with error messages
    
   $frame1_tl = $tl->Frame(
                     -borderwidth => 4,
                     -relief => 'groove',
                    )->pack(
                            -padx => 5,
                            -pady => 5,
                            -side => 'left',
                            -fill => 'both'
                           );  
  
  
    ### Frame 3 to modify lvtool output options

   $frame3_tl = $tl->Frame(
                     -borderwidth => 2,
                     -relief => 'sunken'
                        )->pack(
                            -padx => 5,
                            -pady => 5,
                            -anchor => 'e',
                            -fill => 'both'
                           );
    
   $frame1_tl->Label(
               -font => "Helvetica 18 normal",
               -text => "Possible Error Messages"
              )->pack(
                      -anchor => 'w'
                     );
                         
   $frame1_tl->Label(
               -text => "$explain_error_selection",
               -justify => 'left'
              )->pack(
                      -anchor => 'w'
                     );
   
   
   $lb1 = $frame1_tl->Scrolled( "Listbox", 
   			-scrollbars => "e", 
   			-height => 15, 
  			-selectmode => "extended",
  			-exportselection => 0,
  			-width => 0,
  			-relief => 'sunken',
   			-background => 'white',
   			-highlightcolor => 'blue',
   			-yscrollcommand => ['set' => $yscrollbar],
   		      )->pack(
   		               -side => 'left'
   		               # -fill => 'both'
   		             );
   
   # Frame 2 to display the different types of messages that can appear in a label validation
   # Test.   
   $frame2_tl->Label(
               -font => "Helvetica 18 normal",
               -text => "Possible Warning Messages"
              )->pack(
                      -anchor => 'w'
                     );
                     
   $frame2_tl->Label(
               -text => "$explain_warning_selection",
               -justify => 'left'
              )->pack(
                      -anchor => 'w'
                     );
   
   
   $lb2 = $frame2_tl->Scrolled( "Listbox", 
   			-scrollbars => "e", 
   			-height => 15,
  			-selectmode => "extended",
  			-exportselection => 0,
  			-width => 0,
  			-relief => 'sunken',
   			-background => 'white',
   			-highlightcolor => 'blue',
   			-yscrollcommand => ['set' => $yscrollbar],
   		      )->pack(
   		               -side => 'left',
   		               # -fill => 'both'
   		             );
   
   # Initialize Entry widget with default value of '300' max errors
   $max_err = 300;
   $frame3_tl->LabEntry(-label => "Max Errors Allowed:", 
   		     -textvariable => \$max_err,
   		     -background => 'white',
   		     -labelPack => [ -side => 'left', -anchor => 'w' ]
   		     )->pack(
   		     	     -pady => 15,
   		     	     -side => 'top',
   		     	     -anchor => 'nw');
   
   $frame3_tl->Checkbutton(
                     -text => "Ignore All Warning Messages",
                     -variable => \$no_warnings
                    )->pack(
                            -anchor => 'w',
                            -pady => 15
                           );
                           
   $frame3_tl->Checkbutton(
   	 	     -text => "Ignore Info Messages",
   	 	     -variable => \$no_info
   	            )->pack(
   	            	    -anchor => 'w',
   	            	    -pady => 15
   	            	    );
   
   $l3d = 1;
   $frame3_tl->Checkbutton(
   		     -text => "Validate Only Files That Contain Level 3 Labels",
   		     -variable => \$l3d
   		    )->pack(
   		    	   -anchor => 'w',
   		    	   -pady => 15
   		    	   );
   	            	    
   $frame3_tl->Checkbutton(
   		     -text => "Retrieve/Clear Saved Messages",
   		     -variable => \$clear_logfile,
   		     -command => sub {
   		        if($clear_logfile) {
   		           clear_msgs();
   		        }
   		        else {
   		           retrieve_saved_msgs();
   		        }
   		      }
   		     )->pack(
   		     	    -anchor => 'w',
   		     	    -pady => 15
   		     	    );
   
   # Used for the "Done" button
   
   $frame4_tl = $tl->Frame(
   		      -borderwidth => 4
   		       )->pack(
   		       	      -anchor => 's'
   		       	      );
   
   $frame4_tl->Button(
   		  -text => "Run Tests",
   		  -relief => 'raised',
   		  -padx => 10,
   		  -command => sub { if($listerr == 1) { set_lvt_options();} run_tests();}
   		  )->pack(
   		  	 -pady => 5,
   		  	 -padx => 15,
   		  	 -side => 'left'
   		  	 );
   
   $frame4_tl->Button(
   		 -text => "Back To Main Menu",
   		 -relief => 'raised',
   		 -padx => 10,
   		 -command => sub{ $tl->withdraw(); }
   		 )->pack(
   		 	-pady => 5,
   		 	-padx => 15,
   		 	-side => 'right'
   		 	);
   
   Mainloop;
   
   # Grab the messages in message_list.txt       
   open(MSGFILE, "message_list.txt") || die("can't open message_list.txt file"); 
   
   #First, grab the error message list from the file
   while( chop($line = <MSGFILE>) && ($line !~ /END OF ERROR LIST/) )
   {
   	if($line !~ /POSSIBLE ERROR MESSAGES/)
   	{
   		@errmsgs = split /=\s+/, $line;
   		
   		#display_error_list is the list that the user sees in the GUI
   		
   		#real_error_list is the list that will be used to do the filtering
   		#of unwanted messages
   		push @display_error_list, $errmsgs[0];
   		push @real_error_list, $errmsgs[1];
   	}
   }
   
   #Next, grab the warning message list from the file
   #Skip blank lines first and header for the warnings list
   while( ($line = <MSGFILE>)  && ($line !~ /END OF WARNING LIST/) )
   {
   	#skip blank lines
   	if($line eq "\n") {
   	    next;
   	}
   	
   	chop($line);
   	
   	if($line !~ /POSSIBLE WARNING MESSAGES/)
   	{
   	     @warnmsgs = split /=\s+/, $line;
   	     

   	     push @display_warning_list, $warnmsgs[0];
   	     push @real_warning_list, $warnmsgs[1];
   	}
   }
   
   # insert error and warning lists onto the listbox
   $lb1->insert(0, @display_error_list);  
   $lb2->insert(0, @display_warning_list);
   
   close MSGFILE;
   
   # Check the logfile to see if any messages were saved
   # from a previous run of this tool. If there was, then
   # this will automatically select those messages that the
   # user did not want to see
   
   retrieve_saved_msgs();   
   		 
}   


#Subroutine to call right when the user has finished selecting their desired settings

sub set_lvt_options {
    
    # Enter in more command line options to label validation test
    # if they were selected
    
    $command_opts = $command_opts . " --checklbl" if($l3d);  
    $command_opts = $command_opts . " --nwarn" if($no_warnings);
    $command_opts = $command_opts . " --ninfo" if($no_info);
    $command_opts = $command_opts . " --max " . "$max_err" if($max_err =~ /\d/);
    $command_opts = $command_opts . " --lf ";

    if($clear_logfile) {
    	unlink $logfile;
    }
    
    #Retrieve the error and warning messages the user selected to ignore
    
    @ignore_errs = $lb1->curselection();
    @ignore_warns = $lb2->curselection();
    
    if( ((@ignore_errs + 0) != 0) || ((@ignore_warns + 0) != 0) ) {
       open(LOGFILE, ">$logfile") or die "Can't open logfile";
       
       foreach(@ignore_errs) {
    	  print LOGFILE "$real_error_list[$_]\n";
       }
    
       foreach(@ignore_warns) {
    	  print LOGFILE "$real_warning_list[$_]\n"; 
       }
    
       close LOGFILE;
    }
    else {
    	unlink $logfile;
    }
    
}

# Subroutine to clear all the messages selected in the GUI screen
sub clear_msgs() {
	
    if($clear_logfile) {
    	$lb1->selectionClear(0, "end");
    	$lb2->selectionClear(0, "end");
    }

}

# Look at the logfile and retrieve any messages that were
# saved. The messages that were saved will be highlighted
# in the settings screen.

sub retrieve_saved_msgs() {
	
     if( open(SAVEDMSGS, "<$logfile") ) {
   	while( $line = <SAVEDMSGS> ) {
   		
   		chop ($line);
   		$counter = 0;
   		foreach $msg (@real_error_list) {
   	 	    if( $line eq $msg ) {
   			$lb1->selectionSet($counter);
   			last;
   		    }
   		    ++$counter;
   		}
   		
   		$counter = 0;
   		foreach $msg (@real_warning_list) {
   		    if( $line eq $msg ) {
   		    	$lb2->selectionSet($counter);
   		    	last;
   		    }
   		    ++$counter;
   		}
   		
   	}
   	close SAVEDMSGS;
    }
}

# Cleanup arrays used in the top-level window
sub cleanup_tl_vars() {
	
	@ignore_errs = ();
	@ignore_warns = ();
	@display_error_list = ();
	@display_warning_list = ();
	@real_warning_list = ();
	@real_error_list = ();
}
	